@@ -22,7 +22,7 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
                     leads=NULL,intercs=FALSE,archive=TRUE,sqrts=NULL,
                     lgstc=NULL,noms=NULL,incheck=TRUE,ords=NULL,collect=FALSE,
                     arglist=NULL, priors=NULL,bounds=NULL,
-                    max.resample=1000, overimp = NULL, emburn=NULL) {
+                    max.resample=1000, overimp = NULL, emburn=NULL, boot.type=NULL) {
 
                                         #Checks for errors in list variables
   listcheck<-function(vars,optname) {
@@ -173,13 +173,18 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
   AMp<-ncol(x)
   subbedout<-c(idvars,cs,ts)
 
+  if (is.null(idvars))
+    idcheck <- c(1:AMp)
+  else
+    idcheck <- -idvars
 
-                                        #Error Code: 4
-                                        #Completely missing columns
-  if (any(colSums(!is.na(x)) <= 1)) {
-    all.miss <- colnames(x)[colSums(!is.na(x)) <= 1]
+  ## Error Code: 4
+  ## Completely missing columns
+
+  if (any(colSums(!is.na(x[,idcheck])) <= 1)) {
+    all.miss <- colnames(x[,idcheck])[colSums(!is.na(x[,idcheck])) <= 1]
     if (is.null(all.miss)) {
-      all.miss <- which(colSums(!is.na(x)) <= 1)
+      all.miss <- which(colSums(!is.na(x[,idcheck])) <= 1)
     }
     all.miss <- paste(all.miss, collapse = ", ")
     error.code<-4
@@ -629,14 +634,14 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
   }
 
   if (is.null(c(noms,ords,idvars,cs)))
-    fact<-c(1:AMp)
+    fact <- c(1:AMp)
   else
-    fact<--c(noms,ords,idvars,cs)
+    fact <- -c(noms,ords,idvars,cs)
 
   if (is.null(c(cs,idvars)))
-    idcheck<-c(1:AMp)
+    idcheck <- c(1:AMp)
   else
-    idcheck<--c(cs,idvars)
+    idcheck <- -c(cs,idvars)
 
   ##Error code: 37
   ##factors out of the noms,ids,ords,cs
@@ -698,7 +703,7 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
   }
                                         #Error code: 39
                                         #No missing observation
-  if (!any(is.na(x[,idcheck,drop=FALSE]))) {
+  if (!any(is.na(x[,idcheck,drop=FALSE])) & is.null(overimp)) {
     error.code<-39
     error.mess<-paste("Your data has no missing values.  Make sure the code for \n",
                       "missing data is set to the code for R, which is NA.")
@@ -920,13 +925,15 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
     if (!o.num | !o.size) {
       error.code <- 53
       error.mess <- "The overimp matrix needs to be a two-column numeric matrix."
+      return(list(code=error.code,mess=error.mess))
     }
 
     ## Error 54:
     ## overimp out of range
-    if (!o.rows | ! o.cols) {
+    if (!o.rows | !o.cols) {
       error.code <- 54
-      error.code <- "A row/column pair in overimp is outside the range of the data."
+      error.mess <- "A row/column pair in overimp is outside the range of the data."
+      return(list(code=error.code,mess=error.mess))
     }
 
   }
@@ -937,6 +944,12 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
     }
   }
 
+  if (!is.null(boot.type)) {
+    if (!(boot.type %in% c("ordinary", "none"))) {
+      stop("boot.type must be either 'ordinary' or 'none'")
+    }
+  }
+
   if (is.data.frame(x)) {
     if (sum(sapply(x, length) == 0)) {
       bad.var <- colnames(x)[sapply(x,length) == 0]
@@ -949,5 +962,23 @@ amcheck <- function(x,m=5,p2s=1,frontend=FALSE,idvars=NULL,logs=NULL,
     }
   }
 
+  if (nrow(na.omit(x[,idcheck,drop=FALSE])) > ncol(x[,idcheck,drop=FALSE])) {
+    if (is.data.frame(x)) {
+      lmcheck <- lm(I(rnorm(AMn))~ ., data = x[,idcheck, drop = FALSE])
+    } else {
+      lmcheck <- lm(I(rnorm(AMn))~ ., data = as.data.frame(x[,idcheck, drop = FALSE]))
+    }
+    if (any(is.na(coef(lmcheck)))) {
+      bad.var <- names(coef(lmcheck))[which(is.na(coef(lmcheck)))]
+      if (length(bad.var) == 1) {
+        warning(paste("The variable", bad.var, "is perfectly collinear with another variable in the data.\n"))
+      } else {
+        bad.var <- paste(bad.var, collapse = ", ")
+        warning(paste("The variables (or variable with levels)", bad.var, "are perfectly collinear with another variable in the data.\n"))
+      }
+
+    }
+  }
+
   return(list(m=m,priors=priors))
 }
